# This file has been generated


build_system = 'AppVeyor'
build_id = '29634022'
build_number = '415'
build_version = '1.0.415'
job_id = 'x4mxxasn97s4w95f'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/x4mxxasn97s4w95f'
